mod Mlogicpa

incl Mtex

incl Mtexg

rem 
  \para \bf  Chapter. Peano Arithmetic.  \end 

rem 
  \para \bf  Section. Builting Functions and Predicates.  \end 

rem 
  \para \bf  Addition.  \end The addition function \ft x+y \end is pre-defined 
  in CL. The following properties are built in into the CL theorem prover: 
  \eq* 
    x+y = y+x
  \end
  \eq* 
    x+(y+z) = Tex1_b(x+y)+z
  \end
  \eq* 
    z+x = z+y <-> x = y
  \end
  \eq* 
    x+y = 0 <-> x = 0 & y = 0
  \end

rem 
  \para \bf  Multiplication.  \end The multiplication function \ft x*y \end is 
  pre-defined in CL. The following properties are built in into the CL theorem 
  prover: 
  \eq* 
    x*y = y*x
  \end
  \eq* 
    x*(y*z) = Tex1_b(x*y)*z
  \end
  \eq* 
    x != 0 -> x*y = x*z <-> y = z
  \end
  \eq* 
    x*y = 0 <-> x = 0 \/ y = 0
  \end
  \eq* 
    x*y = 1 <-> x = 1 & y = 1
  \end
  \eq* 
    x*(y+z) = x*y+x*z
  \end
  \eq* 
    1*x = x & 2*x = x+x & 3*x = x+x+x & Tex0_dotsc
  \end

rem 
  \para \bf  Modified subtraction.  \end The modified subtraction function 
  \ft x-y \end is pre-defined in CL. The following properties are built in into 
  the CL theorem prover: 
  \eq* 
    x >= y -> x-y = z <-> x = y+z
  \end
  \eq* 
    x <= y -> x-y = 0
  \end
  \eq* 
    x+y-(x+z) = y-z
  \end
  \eq* 
    x*(y-z) = x*y-x*z
  \end

rem 
  \para \bf  Integer division.  \end The integer division function \ft x/y \end 
  is pre-defined in CL. The following properties are built in into the CL 
  theorem prover: 
  \eq* 
    0/x = 0
  \end
  \eq* 
    x/0 = 0
  \end
  \eq* 
    x/1 = x
  \end
  \eq* 
    x < y -> x/y = 0
  \end
  \eq* 
    x != 0 -> x/x = 1
  \end
  \eq* 
    x*y/(x*z) = x/x*(y/z)
  \end
  \eq* 
    x != 0 -> (x*y+z)/x = y+z/x
  \end

rem 
  \para \bf  Remainder.  \end The remainder function \ft x mod y \end is 
  pre-defined in CL. The following properties are built in into the CL theorem 
  prover: 

rem 
  \para \bf  Comparision predicates.  \end The less-than comparison predicate 
  \ft x <= y \end is pre-defined in CL. The following properties are built in 
  into the CL theorem prover: 
  \eq* 
    x <= x
  \end
  \eq* 
    x <= y & y <= x -> x = y
  \end
  \eq* 
    x <= y & y <= z -> x <= z
  \end
  \eq* 
    0 <= x
  \end
  \eq* 
    x <= 0 <-> x = 0
  \end
  \eq* 
    x+y <= x+z <-> y <= z
  \end
  \eq* 
    x != 0 -> x*y <= x*z <-> y <= z
  \end
  \para Similarly for \ft x < y \end, \ft x >= y \end and \ft x > y \end. 

rem 
  \para \bf  Pairing function.  \end The Catalan pairing function \ft x,y \end 
  is pre-defined in CL. The following properties are built in into the CL 
  theorem prover: 
  \eq* 
    x1,x2 = y1,y2 <-> x1 = y1 & x2 = y2
  \end
  \eq* 
    x,y = 0 <-> \f
  \end
  \para Moreover, we have 
  \eq (occur_check)
    x = Tex2_bappl1(Tex0_tau,x) <-> \f
  \end
  \para for terms \ft Tex2_bappl1(Tex0_tau,x) \end of the form 
  \eq* 
    Tex2_equiv(Tex2_bappl1(Tex0_tau,x),x,y)
  \end
  \eq* 
    Tex2_equiv(Tex2_bappl1(Tex0_tau,x),y,x)
  \end
  \eq* 
    Tex2_equiv(Tex2_bappl1(Tex0_tau,x),(x,y1),y2)
  \end
  \eq* 
    Tex2_equiv(Tex2_bappl1(Tex0_tau,x),(y1,x),y2)
  \end
  \eq* 
    Tex2_equiv(Tex2_bappl1(Tex0_tau,x),y1,x,y2)
  \end
  \eq* 
    Tex2_equiv(Tex2_bappl1(Tex0_tau,x),y1,y2,x)
  \end
  \eq* 
    Tex0_dotsc
  \end

rem 
  \para \bf  Pair size.  \end The pair size function \ft |x| \end is defined in 
  the module \it  Standard  \end. The following its properties are built in into 
  the CL theorem prover: 
  \eq* 
    |0| = 0
  \end
  \eq* 
    |v,w| = |v|+|w|+1
  \end
  \eq* 
    |x| = 0 <-> x = 0
  \end
  \eq* 
    |x| = 1 <-> x = 0,0
  \end
  \eq* 
    |x++y| = |x|+|y|
  \end

rem 
  \para \bf  List concatenation.  \end The list concatenation function 
  \ft x++y \end is defined in the module \it  Standard  \end. The following 
  properties are built in into the CL theorem prover: 
  \eq* 
    0++y = y
  \end
  \eq* 
    (v,w)++y = v,w++y
  \end
  \eq* 
    x++y = 0 <-> x = 0 & y = 0
  \end
  \eq* 
    x++0 = x
  \end
  \eq* 
    x++(y++z) = Tex1_b(x++y)++z
  \end
  \eq* 
    z++x = z++y <-> x = y
  \end
  \eq* 
    x++(a,0) = y++(b,0) <-> x = y & a = b
  \end
  \eq* 
    x++z = y++z <-> x = y
  \end
  \eq* 
    x++y = a,0 -> x = a,0 & y = 0 \/ x = 0 & y = a,0
  \end
  \eq* 
    x++(a,y) = a,0 <-> x = 0 & y = 0
  \end
  \eq* 
    x++a++y = a,0 <-> a = 0 & x++y = 0,0
  \end

rem 
  \para \bf  List membership.  \end The list membership predicate 
  \ft x in y \end is defined in the module \it  Standard  \end. The following 
  properties are built in into the CL theorem prover: 
  \eq* 
    x in 0 <-> \f
  \end
  \eq* 
    x in v,w <-> x = v \/ x in w
  \end
  \eq* 
    x in y++z <-> x in y \/ x in z
  \end

rem 
  \para \bf  Section. Case Analysis.  \end 

rem 
  \para \bf  Monadic case analysis.  \end Syntax: 
  \eq* 
    'case N ; <term>'
  \end
  \para Example(s): 
  \items 
   \item \para \ft Tex_f1_g_x_case Tex_f2_g_x_sc(N,Tex0_tau) \end 
         \eq* 
           Tex_f2_g_x_nrule(Tex_f0_g_x_nothing,
                            Tex_f2_g_x_branch(Tex_f1_g_x_assum Tex_f2_g_eq(Tex0_tau,
                                                                           0),
                                              Tex_f1_g_x_assum Tex_f2_g_eq(Tex0_tau,
                                                                           x+1)))
         \end
         \para where \ft x \end is a new variable \it  (eigen-variable)  \end. 
  \end
  \para Note that the predicate \ft N \end is defined in the module \it  
  Standard  \end. 

rem 
  \para \bf  Dichotomy case analysis.  \end Syntax: 
  \eq* 
    'case Dich ; <term>,<term>'
  \end
  \para Example(s): 
  \items 
   \item \para \ft Tex_f1_g_x_case Tex_f2_g_x_sc(Dich,Tex0_rho,Tex0_tau) \end 
         \eq* 
           Tex_f2_g_x_nrule(Tex_f0_g_x_nothing,
                            Tex_f2_g_x_branch(Tex_f1_g_x_assum Tex_f2_g_le(Tex0_rho,
                                                                           Tex0_tau),
                                              Tex_f1_g_x_assum Tex_f2_g_gt(Tex0_rho,
                                                                           Tex0_tau)))
         \end
  \end
  \para Note that the binary predicate \ft Dich \end is defined in the module 
  \it  Standard  \end. 

rem 
  \para \bf  Trichotomy case analysis.  \end Syntax: 
  \eq* 
    'case Trich ; <term>,<term>'
  \end
  \para Example(s): 
  \items 
   \item \para \ft Tex_f1_g_x_case Tex_f2_g_x_sc(Trich,Tex0_rho,Tex0_tau) \end 
         \eq* 
           Tex_f2_g_x_nrule(Tex_f0_g_x_nothing,
                            Tex_f2_g_x_branch(Tex_f1_g_x_assum Tex_f2_g_lt(Tex0_rho,
                                                                           Tex0_tau),
                                              Tex_f2_g_x_branch(Tex_f1_g_x_assum Tex_f2_g_eq(Tex0_rho,
                                                                                             Tex0_tau),
                                                                Tex_f1_g_x_assum Tex_f2_g_gt(Tex0_rho,
                                                                                             Tex0_tau))))
         \end
  \end
  \para Note that the binary predicate \ft Trich \end is defined in the module 
  \it  Standard  \end. 

rem 
  \para \bf  Section. Induction.  \end 

rem 
  \para \bf  Mathematical induction.  \end Syntax: 
  \eq* 
    'ind N ; <variable> [; <variable(s)>]'
  \end
  \para where the induction formula is formed from the current sequent. 
  \para Example(s): 
  \items 
   \item \para \ft Tex_f1_g_x_ind Tex_f2_g_x_sc(N,x) \end 
         \eq* 
           Tex_f2_g_x_nrule(Tex_f1_g_x_goal Tex2_bappl1(Tex0_phi,x),
                            Tex_f2_g_x_branch(Tex_f1_g_x_goal Tex2_bappl1(Tex0_phi,
                                                                          0),
                                              Tex_f1_g_x_goal Tex_f2_g_imp(Tex2_bappl1(Tex0_phi,
                                                                                       x),
                                                                           Tex2_bappl1(Tex0_phi,
                                                                                       x+
                                                                                       1))))
         \end
   \item \para \ft Tex_f1_g_x_ind Tex_f2_g_x_sc(Tex_f2_g_x_sc(N,x),y,z) \end 
         \eq* 
           Tex_f2_g_x_nrule(Tex_f1_g_x_goal Tex2_bappl1(Tex0_phi,x),
                            Tex_f2_g_x_branch(Tex_f1_g_x_goal Tex2_bappl1(Tex0_phi,
                                                                          0),
                                              Tex_f1_g_x_goal Tex_f2_g_imp(Tex_f2_g_forall(y,
                                                                                           Tex_f2_g_forall(z,
                                                                                                           Tex2_bappl1(Tex0_phi,
                                                                                                                       x))),
                                                                           Tex2_bappl1(Tex0_phi,
                                                                                       x+
                                                                                       1))))
         \end
  \end
  \para Note that the predicate \ft N \end is defined in the module \it  
  Standard  \end. 

rem 
  \para \bf  Complete mathematical induction.  \end Syntax: 
  \eq* 
    'indm <variable> [; <variable(s)>]'
  \end
  \para where the induction formula is formed from the current sequent. 
  \para Example(s): 
  \items 
   \item \para \ft Tex_f1_g_x_indm(x) \end 
         \eq* 
           Tex_f2_g_x_nrule(Tex_f1_g_x_goal Tex2_bappl1(Tex0_phi,x),
                            Tex_f1_g_x_goal Tex_f2_g_imp(Tex_f2_g_forall(y,
                                                                         Tex_f2_g_imp(Tex_f2_g_lt(y,
                                                                                                  x),
                                                                                      Tex2_bappl1(Tex0_phi,
                                                                                                  y))),
                                                         Tex2_bappl1(Tex0_phi,x)))
         \end
   \item \para \ft Tex_f1_g_x_indm Tex_f2_g_x_sc(x,a,b) \end 
         \eq* 
           Tex_f2_g_x_nrule(Tex_f1_g_x_goal Tex2_bappl1(Tex0_phi,x),
                            Tex_f1_g_x_goal Tex_f2_g_imp(Tex_f2_g_forall(y,
                                                                         Tex_f2_g_imp(Tex_f2_g_lt(y,
                                                                                                  x),
                                                                                      Tex_f2_g_forall(a,
                                                                                                      Tex_f2_g_forall(b,
                                                                                                                      Tex2_bappl1(Tex0_phi,
                                                                                                                                  y))))),
                                                         Tex2_bappl1(Tex0_phi,x)))
         \end
  \end

rem 
  \para \bf  Course of values induction with measure.  \end Syntax: 
  \eq* 
    'indm <measure> [; <variable(s)>]'
  \end
  \para where the induction formula is formed from the current sequent. 
  \para Example(s): 
  \items 
   \item \para \ft Tex_f1_g_x_indm Tex2_bappl1(Tex0_mu,x) \end 
         \eq* 
           Tex_f2_g_x_nrule(Tex_f1_g_x_goal Tex2_bappl1(Tex0_phi,x),
                            Tex_f1_g_x_goal Tex_f2_g_imp(Tex_f2_g_forall(y,
                                                                         Tex_f2_g_imp(Tex_f2_g_lt(Tex2_bappl1(Tex0_mu,
                                                                                                              y),
                                                                                                  Tex2_bappl1(Tex0_mu,
                                                                                                              x)),
                                                                                      Tex2_bappl1(Tex0_phi,
                                                                                                  y))),
                                                         Tex2_bappl1(Tex0_phi,x)))
         \end
   \item \para 
         \ft Tex_f1_g_x_indm Tex_f2_g_x_sc(Tex2_bappl1(Tex0_mu,x),a,b) \end 
         \eq* 
           Tex_f2_g_x_nrule(Tex_f1_g_x_goal Tex2_bappl1(Tex0_phi,x),
                            Tex_f1_g_x_goal Tex_f2_g_imp(Tex_f2_g_forall(y,
                                                                         Tex_f2_g_imp(Tex_f2_g_lt(Tex2_bappl1(Tex0_mu,
                                                                                                              y),
                                                                                                  Tex2_bappl1(Tex0_mu,
                                                                                                              x)),
                                                                                      Tex_f2_g_forall(a,
                                                                                                      Tex_f2_g_forall(b,
                                                                                                                      Tex2_bappl1(Tex0_phi,
                                                                                                                                  y))))),
                                                         Tex2_bappl1(Tex0_phi,x)))
         \end
  \end

rem 
  \para \bf  Section. Auxiliary Commands.  \end 

rem 
  \para \bf  Command \it  open  \end.  \end To force opening the definitions of 
  defined functions and predicates try to use the pseudo-command \it  open 
   \end. The system gives you the full list of commands available in the current 
  context which are needed to evaluate defined functions and predicates. 

